// src/hoist-static/transfrom.ts
var transformHoistStatic = (ctx) => {
  function moveToScript(decl, prefix = "") {
    const text = s.slice(startOffset + decl.start, startOffset + decl.end);
    s.remove(startOffset + decl.start, startOffset + decl.end);
    ctx.scriptCode.prepend += `
${prefix}${text}`;
  }
  const { sfc, s } = ctx;
  if (!sfc.scriptSetup)
    return;
  const startOffset = sfc.scriptSetup.loc.start.offset;
  for (const stmt of sfc.scriptCompiled.scriptSetupAst) {
    if (stmt.type === "VariableDeclaration" && stmt.kind === "const") {
      const decls = stmt.declarations;
      let count = 0;
      for (const [i, decl] of decls.entries()) {
        if (!decl.init || !isLiteral(decl.init))
          continue;
        count++;
        moveToScript(decl, "const ");
        if (decls.length > 1) {
          const isLast = i === decls.length - 1;
          const start = isLast ? decls[i - 1].end : decl.end;
          const end = isLast ? decl.start : decls[i + 1].start;
          s.remove(startOffset + start, startOffset + end);
        }
      }
      if (count === decls.length) {
        s.remove(startOffset + stmt.start, startOffset + stmt.end);
      }
    } else if (stmt.type === "TSEnumDeclaration") {
      const isAllConstant = stmt.members.every(
        (member) => !member.initializer || isLiteral(member.initializer)
      );
      if (!isAllConstant)
        continue;
      moveToScript(stmt);
    }
  }
};
function isLiteral(node) {
  var _a;
  if ((_a = node.leadingComments) == null ? void 0 : _a.some(
    (comment) => comment.value.trim() === "hoist-static"
  )) {
    return true;
  }
  switch (node.type) {
    case "UnaryExpression":
      return isLiteral(node.argument);
    case "BinaryExpression":
      return isLiteral(node.left) && isLiteral(node.right);
    case "ArrayExpression":
      return node.elements.every((element) => element && isLiteral(element));
    case "ObjectExpression":
      return node.properties.every(
        (prop) => prop.type === "ObjectProperty" && (isLiteral(prop.key) || !prop.computed && prop.key.type === "Identifier") && isLiteral(prop.value)
      );
    case "SequenceExpression":
    case "TemplateLiteral":
      return node.expressions.every((expr) => isLiteral(expr));
  }
  if (isLiteralType(node))
    return true;
  return false;
}
function isLiteralType(node) {
  return node.type.endsWith("Literal");
}

export {
  transformHoistStatic
};
