import {
  transformHoistStatic
} from "./chunk-NTWFP6CB.mjs";
import {
  finalizeContext,
  getTransformResult,
  initContext,
  transformDefineModel
} from "./chunk-PKMJSYMD.mjs";

// src/index.ts
import { createUnplugin } from "unplugin";
import { createFilter } from "@rollup/pluginutils";
import { getPackageInfoSync } from "local-pkg";
import { transform as transformDefineOptions } from "unplugin-vue-define-options";
function resolveOption(options) {
  let version = options.version;
  if (version === void 0) {
    const vuePkg = getPackageInfoSync("vue");
    if (vuePkg) {
      version = +vuePkg.version.slice(0, 1);
    } else {
      version = 3;
    }
  }
  return {
    include: options.include || [/\.vue$/, /\.vue\?vue/],
    exclude: options.exclude || void 0,
    version,
    defineOptions: options.defineOptions ?? true,
    defineModel: options.defineModel ?? true,
    hoistStatic: options.defineModel ?? true
  };
}
var src_default = createUnplugin((userOptions = {}) => {
  const options = resolveOption(userOptions);
  const filter = createFilter(options.include, options.exclude);
  const name = "unplugin-vue-macros";
  return {
    name,
    enforce: "pre",
    transformInclude(id) {
      return filter(id);
    },
    transform(code, id) {
      try {
        const { ctx, getMagicString } = initContext(code, id);
        if (options.hoistStatic) {
          transformHoistStatic(ctx);
        }
        if (options.defineOptions) {
          transformDefineOptions(ctx);
        }
        if (options.defineModel) {
          transformDefineModel(ctx, options.version);
        }
        finalizeContext(ctx);
        return getTransformResult(getMagicString(), id);
      } catch (err) {
        this.error(`${name} ${err}`);
      }
    }
  };
});

export {
  src_default
};
